class ProvidePhoneNumber {
	modal;
	laterModal;
	providePhoneNumberLaterButton;
	phoneNumberInput;
	agovInputPhoneNumberInput;
	repeatPhoneNumberInput;
	agovInputRepeatPhoneNumberInput;

	constructor() {
		this.modal = document.querySelector('#modal');
		this.laterModal = document.querySelector('#laterModal');
		this.providePhoneNumberLaterButton = document.querySelector('#providePhoneNumberLaterButton');
		this.phoneNumberInput = document.querySelector('#phoneNumberInput');
		this.agovInputPhoneNumberInput = document.querySelector('#agovInputPhoneNumberInput');
		this.agovInputRepeatPhoneNumberInput = document.querySelector('#agovInputRepeatPhoneNumberInput');
		this.repeatPhoneNumberInput = document.querySelector('#repeatPhoneNumberInput');

		document.querySelector('#laterCheckbox').checked = true;
		this.initializePhoneInput(this.phoneNumberInput);
		this.addPhoneInputEventHandlers(this.phoneNumberInput);

		this.providePhoneNumberLaterButton.addEventListener('click', () => {
			this.setVisible(this.laterModal);
		});
		document.querySelector('#laterModalBack').addEventListener('click', () => {
			this.resetValidation(this.agovInputPhoneNumberInput);
			this.setInvisible(this.laterModal);
		});
		document.querySelector('#repeatPhoneNumberModalBack').addEventListener('click', () => {
			this.initializePhoneInput(this.phoneNumberInput);
			this.addPhoneInputEventHandlers(this.phoneNumberInput);
			this.setInvisible(this.modal);
		});
		document.querySelector('#repeatPhoneNumberModalContinue').addEventListener('click', () => {
			if (this.validateInput(this.agovInputRepeatPhoneNumberInput)) {
				this.evaluatePhoneNumbersAndSubmit();
				this.initializePhoneInput(this.phoneNumberInput);
				this.addPhoneInputEventHandlers(this.phoneNumberInput);
			}
		});
		document.querySelector('#providePhoneNumberContinueButton').addEventListener('click', () => {
			if (this.validateInput(this.agovInputPhoneNumberInput)) {
				this.repeatPhoneNumberInput.value = '';
				this.initializePhoneInput(this.repeatPhoneNumberInput);
				this.addPhoneInputEventHandlers(this.repeatPhoneNumberInput);
				this.resetValidation(this.agovInputRepeatPhoneNumberInput);
				this.showErrorBanner(false);
				this.setVisible(this.modal);
			}
		});
	}

	addPhoneInputEventHandlers(phoneInputElement) {
		phoneInputElement.addEventListener('input', () => {
			this.formatAndEmitPhoneNumberValue(phoneInputElement);
		});
		phoneInputElement.addEventListener('countrychange', () => {
			this.onPhoneNumberCountryChange(phoneInputElement);
		});
	}

	initializePhoneInput(inputElement) {
		window.phoneNumberUtils.initializePhoneNumberField(inputElement);
		if (inputElement.value) {
			window.phoneNumberUtils.setNumber(inputElement.value);
			this.formatAndEmitPhoneNumberValue(inputElement);
		}
		else {
			window.phoneNumberUtils.formatAfterCountryChange();
			inputElement.value = window.phoneNumberUtils.getInputValue();
		}
	}

	onPhoneNumberCountryChange(inputElement) {
		inputElement.value = window.phoneNumberUtils.formatAfterCountryChange();
	}

	formatAndEmitPhoneNumberValue(inputElement) {
		inputElement.value = window.phoneNumberUtils.formatUsingInputValue();
	}

	resetValidation(agovInput) {
		agovInput.validity = true;
	}

	validateInput(agovInput) {
		const innerInput = agovInput.querySelector('input');
		if (innerInput.value.trim().length === 0) {
			agovInput.supportingText = 0;
			agovInput.focus();
			agovInput.validity = false;
			return false;
		}
		else if (!window.phoneNumberUtils.validatePhoneNumber(false, true)) {
			agovInput.supportingText = 2;
			agovInput.focus();
			agovInput.validity = false;
			return false;
		}
		return true;
	}

	setInvisible(modalElement) {
		window.utils.changeDisplay(modalElement, 'hidden', 'block');
	}

	setVisible(modalElement) {
		window.utils.changeDisplay(modalElement, 'block', 'hidden');
		const isMobile = !!/(iPhone|Android)/.test(window.navigator.userAgent);
		if (isMobile) {
			modalElement.classList.add('mobileDrawer');
			document.dispatchEvent(new Event('initDrawerPopup'));
		}
	}

	arePhoneNumbersEqual() {
		const phoneNumber = this.phoneNumberInput.value.trim();
		const repeatedPhoneNumber = this.repeatPhoneNumberInput.value.trim();
		return phoneNumber === repeatedPhoneNumber;
	}

	evaluatePhoneNumbersAndSubmit() {
		if (this.arePhoneNumbersEqual()) {
			document.querySelector('#mobile').value = this.repeatPhoneNumberInput.value.trim().replaceAll(' ', '');
			document.querySelector('#submit').click();
		}
		else {
			this.setInvisible(this.modal);
			this.showErrorBanner(true);
		}
	}

	showErrorBanner(visible) {
		const errorBanner = document.getElementById('errorBanner');
		if (visible) {
			window.utils.changeDisplay(errorBanner, 'block', 'hidden');
		}
		else {
			window.utils.changeDisplay(errorBanner, 'hidden', 'block');
		}
	}
}

document.addEventListener('DOMContentLoaded', () => {
	document.dispatchEvent(new Event('initPhoneNumberUtils'));
	window.providePhoneNumber = new ProvidePhoneNumber();
});
